from sqlmodel import Field, SQLModel, Relationship

from gisaf.models.metadata import gisaf_admin_table_args


class UserRoleLink(SQLModel, table=True):
    __tablename__ = 'roles_users'
    __table_args__ = gisaf_admin_table_args
    user_id: int | None = Field(
        default=None,
        foreign_key=gisaf_admin_table_args['schema'] + '.user.id',
        primary_key=True
    )
    role_id: int | None = Field(
        default=None,
        foreign_key=gisaf_admin_table_args['schema'] + '.role.id',
        primary_key=True
    )


class UserBase(SQLModel):
    username: str
    email: str
    disabled: bool | None = False


class User(UserBase, table=True):
    __table_args__ = gisaf_admin_table_args
    id: int | None = Field(default=None, primary_key=True)
    roles: list["Role"] = Relationship(back_populates="users",
                                       link_model=UserRoleLink)
    password: str | None = None

    def can_view(self, model) -> bool:
        if hasattr(model, 'viewable_role'):
            return model.viewable_role in (role.name for role in self.roles)
        else:
            return True


class RoleBase(SQLModel):
    name: str = Field(unique=True)

class RoleWithDescription(RoleBase):
    description: str | None

class Role(RoleWithDescription, table=True):
    __table_args__ = gisaf_admin_table_args
    id: int | None = Field(default=None, primary_key=True)
    users: list[User] = Relationship(back_populates="roles",
                                     link_model=UserRoleLink)


class UserReadNoRoles(UserBase):
    id: int
    email: str | None


class RoleRead(RoleBase):
    id: int
    users: list[UserReadNoRoles] = []


class RoleReadNoUsers(RoleBase):
    id: int


class UserRead(UserBase):
    id: int
    email: str | None
    roles: list[RoleReadNoUsers] = []